@@ -34,8 +34,9 @@ class GM_Import {
 <form name="stepOne" method="get">
 <input type="hidden" name="import" value="greymatter" />
 <input type="hidden" name="step" value="1" />
+<?php wp_nonce_field('import-greymatter'); ?>
 <h3><?php _e('Second step: GreyMatter details:') ?></h3>
-<p><table cellpadding="0">
+<table class="form-table">
 <tr>
 <td><?php _e('Path to GM files:') ?></td>
 <td><input type="text" style="width:300px" name="gmpath" value="/home/my/site/cgi-bin/greymatter/" /></td>
@@ -45,17 +46,14 @@ class GM_Import {
 <td><input type="text" style="width:300px" name="archivespath" value="/home/my/site/cgi-bin/greymatter/archives/" /></td>
 </tr>
 <tr>
-<td colspan="2"><br /><?php _e("This importer will search for files 00000001.cgi to 000-whatever.cgi,<br />so you need to enter the number of the last GM post here.<br />(if you don't know that number, just log into your FTP and look it out<br />in the entries' folder)") ?></td>
-</tr>
-<tr>
 <td><?php _e("Last entry's number:") ?></td>
-<td><input type="text" name="lastentry" value="00000001" /></td>
+<td><input type="text" name="lastentry" value="00000001" /><br />
+	<?php _e("This importer will search for files 00000001.cgi to 000-whatever.cgi,<br />so you need to enter the number of the last GM post here.<br />(if you don't know that number, just log into your FTP and look it out<br />in the entries' folder)") ?></td>
 </tr>
 </table>
 </p>
-<p><?php _e("When you're ready, click OK to start importing: ") ?><input type="submit" name="submit" value="<?php _e('OK') ?>" class="search" /></p>
+<p><input type="submit" name="submit" value="<?php _e('Start Importing') ?>" class="button" /></p>
 </form>
-<p>&nbsp</p>
 <?php
 		$this->footer();
 	}
@@ -87,11 +85,13 @@ class GM_Import {
 		}
 
 		if (!chdir($archivespath))
-			wp_die(sprintf(__("Wrong path, %s\ndoesn't exist\non the server"), $archivespath));
+			wp_die(__("Wrong path, the path to the GM entries does not exist on the server"));
 
 		if (!chdir($gmpath))
-			wp_die(sprintf(__("Wrong path, %s\ndoesn't exist\non the server"), $gmpath));
-			
+			wp_die(__("Wrong path, the path to the GM files does not exist on the server"));
+
+		$lastentry = (int) $lastentry;
+
 		$this->header();
 ?>
 <p><?php _e('The importer is running...') ?></p>
@@ -128,7 +128,7 @@ class GM_Import {
 		$user_info = array("user_login"=>"$user_login", "user_pass"=>"$pass1", "user_nickname"=>"$user_nickname", "user_email"=>"$user_email", "user_url"=>"$user_url", "user_ip"=>"$user_ip", "user_domain"=>"$user_domain", "user_browser"=>"$user_browser", "dateYMDhour"=>"$user_joindate", "user_level"=>"1", "user_idmode"=>"nickname");
 		$user_id = wp_insert_user($user_info);
 		$this->gmnames[$userdata[0]] = $user_id;
-		
+
 		printf('<li>'.__('user %s...').' <strong>'.__('Done').'</strong></li>', "<em>$user_login</em>");
 	}
 
@@ -213,23 +213,25 @@ class GM_Import {
 					$user_email=$wpdb->escape("user@deleted.com");
 					$user_url=$wpdb->escape("");
 					$user_joindate=$wpdb->escape($user_joindate);
-					
+
 					$user_info = array("user_login"=>$user_login, "user_pass"=>$pass1, "user_nickname"=>$user_nickname, "user_email"=>$user_email, "user_url"=>$user_url, "user_ip"=>$user_ip, "user_domain"=>$user_domain, "user_browser"=>$user_browser, "dateYMDhour"=>$user_joindate, "user_level"=>0, "user_idmode"=>"nickname");
 					$user_id = wp_insert_user($user_info);
 					$this->gmnames[$postinfo[1]] = $user_id;
-					
+
 					echo ': ';
 					printf(__('registered deleted user %s at level 0 '), "<em>$user_login</em>");
 				}
-			
+
 				if (array_key_exists($postinfo[1], $this->gmnames)) {
 					$post_author = $this->gmnames[$postinfo[1]];
 				} else {
 					$post_author = $user_id;
 				}
-			
+
 				$postdata = compact('post_author', 'post_date', 'post_date_gmt', 'post_content', 'post_title', 'post_excerpt', 'post_status', 'comment_status', 'ping_status', 'post_modified', 'post_modified_gmt');
 				$post_ID = wp_insert_post($postdata);
+				if ( is_wp_error( $post_ID ) )
+					return $post_ID;
 			}
 
 			$c=count($entry);
@@ -267,23 +269,25 @@ class GM_Import {
 				}
 				if ($numAddedComments > 0) {
 					echo ': ';
-					printf(__('imported %d comment(s)'), $numAddedComments);
+				printf( __ngettext('imported %s comment', 'imported %s comments', $numAddedComments) , $numAddedComments);
 				}
 				$preExisting = $numComments - numAddedComments;
 				if ($preExisting > 0) {
 					echo ' ';
-					printf(__('ignored %d pre-existing comments'), $preExisting);
+					printf( __ngettext( 'ignored %s pre-existing comment', 'ignored %s pre-existing comments', $preExisting ) , $preExisting);
 				}
 			}
 			echo '... <strong>'.__('Done').'</strong></li>';
 		}
 	}
+	do_action('import_done', 'greymatter');
 	?>
 </ul><strong><?php _e('Done') ?></strong></li></ul>
 <p>&nbsp;</p>
 <p><?php _e('Completed GreyMatter import!') ?></p>
 <?php
 	$this->footer();
+	return;
 	}
 
 	function dispatch() {
@@ -297,7 +301,10 @@ class GM_Import {
 				$this->greet();
 				break;
 			case 1:
-				$this->import();
+				check_admin_referer('import-greymatter');
+				$result = $this->import();
+				if ( is_wp_error( $result ) )
+					echo $result->get_error_message();
 				break;
 		}
 	}
@@ -309,5 +316,5 @@ class GM_Import {
 
 $gm_import = new GM_Import();
 
-register_importer('greymatter', __('GreyMatter'), __('Import users, posts, and comments from a Greymatter blog'), array ($gm_import, 'dispatch'));
+register_importer('greymatter', __('GreyMatter'), __('Import users, posts, and comments from a Greymatter blog.'), array ($gm_import, 'dispatch'));
 ?>
