@@ -71,7 +71,7 @@ class RSS_Import {
 				$cat_index++;
 			}
 
-			preg_match('|<guid.+?>(.*?)</guid>|is', $post, $guid);
+			preg_match('|<guid.*?>(.*?)</guid>|is', $post, $guid);
 			if ($guid)
 				$guid = $wpdb->escape(trim($guid[1]));
 			else
@@ -110,6 +110,8 @@ class RSS_Import {
 				_e('Post already imported');
 			} else {
 				$post_id = wp_insert_post($post);
+				if ( is_wp_error( $post_id ) )
+					return $post_id;
 				if (!$post_id) {
 					_e("Couldn't get post ID");
 					return;
@@ -135,8 +137,11 @@ class RSS_Import {
 
 		$this->file = $file['file'];
 		$this->get_posts();
-		$this->import_posts();
+		$result = $this->import_posts();
+		if ( is_wp_error( $result ) )
+			return $result;
 		wp_import_cleanup($file['id']);
+		do_action('import_done', 'rss');
 
 		echo '<h3>';
 		printf(__('All done. <a href="%s">Have fun!</a>'), get_option('home'));
@@ -156,7 +161,10 @@ class RSS_Import {
 				$this->greet();
 				break;
 			case 1 :
-				$this->import();
+				check_admin_referer('import-upload');
+				$result = $this->import();
+				if ( is_wp_error( $result ) )
+					echo $result->get_error_message();
 				break;
 		}
 
@@ -170,5 +178,5 @@ class RSS_Import {
 
 $rss_import = new RSS_Import();
 
-register_importer('rss', __('RSS'), __('Import posts from an RSS feed'), array ($rss_import, 'dispatch'));
+register_importer('rss', __('RSS'), __('Import posts from an RSS feed.'), array ($rss_import, 'dispatch'));
 ?>
