@@ -1,5 +1,5 @@
 <?php wp_reset_vars(array('action', 'standalone', 'option_group_id')); ?>
 
 <?php if (isset($_GET['updated'])) : ?>
-<div id="message" class="updated fade"><p><strong><?php _e('Options saved.') ?></strong></p></div>
+<div id="message" class="updated fade"><p><strong><?php _e('Settings saved.') ?></strong></p></div>
 <?php endif; ?>
\ No newline at end of file
