@@ -8,7 +8,7 @@ if ( !file_exists( dirname(__FILE__) . '/wp-config.php') ) {
 	require_once( dirname(__FILE__) . '/wp-includes/classes.php');
 	require_once( dirname(__FILE__) . '/wp-includes/functions.php');
 	require_once( dirname(__FILE__) . '/wp-includes/plugin.php');
-	wp_die("There doesn't seem to be a <code>wp-config.php</code> file. I need this before we can get started. Need more help? <a href='http://codex.wordpress.org/Editing_wp-config.php'>We got it</a>. You can <a href='{$path}setup-config.php'>create a <code>wp-config.php</code> file through a web interface</a>, but this doesn't work for all server setups. The safest way is to manually create the file.", "WordPress &rsaquo; Error");
+	wp_die("There doesn't seem to be a <code>wp-config.php</code> file. I need this before we can get started. Need more help? <a href='http://codex.wordpress.org/Editing_wp-config.php'>We got it</a>. You can create a <code>wp-config.php</code> file through a web interface, but this doesn't work for all server setups. The safest way is to manually create the file.</p><p><a href='{$path}setup-config.php' class='button'>Create a Configuration File</a>", "WordPress &rsaquo; Error");
 }
 
 $wp_did_header = true;
@@ -16,7 +16,6 @@ $wp_did_header = true;
 require_once( dirname(__FILE__) . '/wp-config.php');
 
 wp();
-gzip_compression();
 
 require_once(ABSPATH . WPINC . '/template-loader.php');
 
