@@ -6,7 +6,7 @@ Description: This is not just a plugin, it symbolizes the hope and enthusiasm of
 Author: Matt Mullenweg
 Version: 1.5
 Author URI: http://photomatt.net/
-*/ 
+*/
 
 // These are the lyrics to Hello Dolly
 $lyrics = "Hello, Dolly
@@ -59,10 +59,11 @@ function dolly_css() {
 	#dolly {
 		position: absolute;
 		top: 2.3em;
-margin: 0; padding: 0;
-		right: 1em;
+		margin: 0;
+		padding: 0;
+		right: 10px;
 		font-size: 16px;
-		color: #f1f1f1;
+		color: #d54e21;
 	}
 	</style>
 	";
