@@ -10,7 +10,7 @@ Template Name: Links
 
 <h2>Links:</h2>
 <ul>
-<?php get_links_list(); ?>
+<?php wp_list_bookmarks(); ?>
 </ul>
 
 </div>
