@@ -793,12 +793,12 @@ class WP_Rewrite {
 		$robots_rewrite = array('robots.txt$' => $this->index . '?robots=1');
 
 		//Default Feed rules - These are require to allow for the direct access files to work with permalink structure starting with %category%
-		$default_feeds = array(	'wp-atom.php$'	=>	$this->index .'?feed=atom',
-								'wp-rdf.php$'	=>	$this->index .'?feed=rdf',
-								'wp-rss.php$'	=>	$this->index .'?feed=rss',
-								'wp-rss2.php$'	=>	$this->index .'?feed=rss2',
-								'wp-feed.php$'	=>	$this->index .'?feed=feed',
-								'wp-commentsrss2.php$'	=>	$this->index . '?feed=rss2&withcomments=1');
+		$default_feeds = array(	'.*/wp-atom.php$'	=>	$this->index .'?feed=atom',
+								'.*/wp-rdf.php$'	=>	$this->index .'?feed=rdf',
+								'.*/wp-rss.php$'	=>	$this->index .'?feed=rss',
+								'.*/wp-rss2.php$'	=>	$this->index .'?feed=rss2',
+								'.*/wp-feed.php$'	=>	$this->index .'?feed=feed',
+								'.*/wp-commentsrss2.php$'	=>	$this->index . '?feed=rss2&withcomments=1');
 
 		// Post
 		$post_rewrite = $this->generate_rewrite_rules($this->permalink_structure, EP_PERMALINK);
@@ -993,6 +993,8 @@ class WP_Rewrite {
 
 		// Enable generic rules for pages if permalink structure doesn't begin with a wildcard.
 		$structure = ltrim($this->permalink_structure, '/');
+		if ( $this->using_index_permalinks() )
+			$structure = ltrim($this->permalink_structure, $this->index . '/');
 		if ( 0 === strpos($structure, '%postname%') ||
 			 0 === strpos($structure, '%category%') ||
 			 0 === strpos($structure, '%tag%') ||
